package org.tinygroup.eventnet;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.jboss.netty.channel.ChannelHandler;
import org.jboss.netty.handler.codec.serialization.ClassResolvers;
import org.jboss.netty.handler.codec.serialization.ObjectDecoder;
import org.jboss.netty.handler.codec.serialization.ObjectEncoder;
import org.tinygroup.event.Event;
import org.tinygroup.net.Client;
import org.tinygroup.net.ClientHandler;

public class EventClient extends Client {
	/**
	 * String:EventId
	 * 
	 */
	Map<String, Thread> threadMap = new ConcurrentHashMap<String, Thread>();
	Map<String, Event> eventMap = new ConcurrentHashMap<String, Event>();

	public void updateEvent(String eventId, Event event) {
		eventMap.put(eventId, event);
	}

	public EventClient(String host, int port) {
		super(host, port);
	}

	Event getEvent(String eventId) {
		return eventMap.get(eventId);
	}

	@Override
	public ChannelHandler getHandler() {
		ClientHandler clientHandler = new EventClientHandler();
		clientHandler.setClient(this);
		return clientHandler;
	}

	@Override
	public ChannelHandler getEncoder() {
		ObjectEncoder objectEncoder = new ObjectEncoder();
		return objectEncoder;
	}

	void clearRequest(String eventId) {
		threadMap.remove(eventId);
		eventMap.remove(eventId);
	}

	@Override
	public ChannelHandler getDecoder() {
		return new ObjectDecoder(ClassResolvers.cacheDisabled(null));
	}

	@SuppressWarnings("unchecked")
	@Override
	public <T> T sendObject(Object data) {
		if (!isReady()) {
			throw new RuntimeException("客户端没有就绪，不能发送报文！");
		}
		Event event = (Event) data;
		String eventId = event.getEventId();
		if (isReady()) {
			// 存放数据
			threadMap.put(eventId, Thread.currentThread());
			eventMap.put(eventId, event);
			try {
				context.getChannel().write(data);// 写出数据
				synchronized (eventId) {
					eventId.wait(timeout);
				}
				// 已经醒来
				Event e = getEvent(eventId);
				if (e.getType() == Event.EVENT_TYPE_REQUEST) {// 如果超时，则返回请求超时异常
					throw new RuntimeException("请求" + eventId + "超时");
				}
				Throwable throwable = e.getThrowable();
				if (throwable != null) {// 如果有异常发生，则抛出异常
					if (throwable instanceof RuntimeException) {
						throw (RuntimeException) throwable;
					} else if (throwable instanceof Error) {
						throw (Error) throwable;
					} else {
						throw new RuntimeException(throwable);
					}
				}
				return (T) event;
			} catch (InterruptedException e) {
				throw new RuntimeException(e);
			} finally {
				clearRequest(eventId);
			}
		} else {
			throw new RuntimeException("尚未连接成功");
		}
	}

}
