/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.eventnet;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.jboss.netty.channel.ChannelHandlerContext;
import org.tinygroup.cepcore.CEPCore;
import org.tinygroup.event.Event;
import org.tinygroup.net.ServerHandler;
import org.tinygroup.springutil.SpringUtil;

public class EventServerHandler extends ServerHandler {
	Map<String, Event> eventMap = new ConcurrentHashMap<String, Event>();// 存放要处理的事件
	Map<String, ChannelHandlerContext> channelHandlerContextMap = new ConcurrentHashMap<String, ChannelHandlerContext>();// 存放连接上下文
	ExecutorService executorService = Executors.newFixedThreadPool(Runtime
			.getRuntime().availableProcessors() * 2);

	@Override
	protected void processObject(Object message, ChannelHandlerContext ctx) {
		// 先记录
		Event event = (Event) message;
		String eventId = event.getEventId();
		eventMap.put(eventId, event);
		channelHandlerContextMap.put(eventId, ctx);
		// 获取处理器进行处理
		EventProcessor eventProcessor = new EventProcessor(event);
		executorService.execute(eventProcessor);
	}

	public void clearRequest(String eventId) {
		eventMap.remove(eventId);
		channelHandlerContextMap.remove(eventId);
	}

	class EventProcessor implements Runnable {
		private Event event;

		public EventProcessor(Event event) {
			this.event = event;
		}

		public void run() {
			ChannelHandlerContext channelHandlerContext = channelHandlerContextMap
					.get(event.getEventId());
			if (event.getMode() == Event.EVENT_MODE_ASYNCHRONOUS) {
				// 如果是同步模式，先返回结果
				event.setType(Event.EVENT_TYPE_RESPONSE);
				channelHandlerContext.getChannel().write(
						getAsynchronousResponseEvent(event));
			}
			// TODO 通过本地通道去执行，现在是修改一下状态，就返回
			CEPCore core = SpringUtil.getBean(CEPCore.CEP_CORE_BEAN);
			core.process(event);
			if (event.getMode() == Event.EVENT_MODE_SYNCHRONOUS) {
				event.setType(Event.EVENT_TYPE_RESPONSE);
				channelHandlerContext.getChannel().write(event);
			}
			clearRequest(event.getEventId());

		}

		private Object getAsynchronousResponseEvent(Event event) {
			Event response = new Event();
			response.setEventId(event.getEventId());
			response.setMode(Event.EVENT_MODE_ASYNCHRONOUS);
			response.setType(Event.EVENT_TYPE_RESPONSE);
			return response;
		}

	}
}
